@@ -2,7 +2,7 @@ package Resource::Silo::Container;
 
 use strict;
 use warnings;
-our $VERSION = '0.1203';
+our $VERSION = '0.13';
 
 =head1 NAME
 
@@ -163,30 +163,42 @@ sub _silo_instantiate_res {
 
 # use instead of delete $self->{-cache}{$name}
 sub _silo_cleanup_res {
-    my ($self, $name, @list) = @_;
+    my ($self, $name, %opt) = @_;
 
     # TODO Do we need to validate arguments here?
     my $spec = $self->{-spec}{resource}{$name};
 
+    return if $opt{fork} and $spec->{fork_safe};
+
+    # NOTE Be careful! cleanup must never ever die!
+
     my $action;
     if (!$self->{-override}{$name}) {
         # 1) skip resources that have overrides
         # 2) if we're in "no pid" mode, use fork_cleanup if available
-        $action = $self->{-pid} != $$
+        $action = $opt{fork}
             && $spec->{fork_cleanup}
             || $spec->{cleanup};
     };
     my $known = $self->{-cache}{$name};
 
-    @list = keys %$known
-        unless @list;
+    my @list = keys %$known;
 
-    foreach my $arg (@list) {
-        $arg //= '';
-        next unless defined $known->{$arg};
-        $action->($known->{$arg}) if $action;
-        delete $known->{$arg};
+    if ($action) {
+        foreach my $arg (@list) {
+            local $@; # don't pollute $@ if we're in destructor after an exception
+            eval {
+                $action->($known->{$arg});
+                1;
+            } or do {
+                my $err = $@;
+                Carp::cluck "Failed to cleanup resource '$name/$arg', but trying to continue: $err";
+            };
+        };
     };
+
+    # This will trigger the normal destructor(s) on resource instances, if any
+    delete $self->{-cache}{$name};
 };
 
 # We must create resource accessors in this package
@@ -326,6 +338,8 @@ sub override {
     my ($self, %subst) = @_;
 
     $$self->_silo_check_overrides(\%subst);
+    $self->_cleanup( fork => 1 )
+        if $$ != $$self->{-pid};
     $$self->_silo_do_override(\%subst);
 
     return $self;
@@ -396,7 +410,18 @@ Typically only useful for destruction.
 =cut
 
 sub cleanup {
-    my $self = ${ $_[0] };
+    my $self = shift;
+    # Don't give the user access to options (yet)
+
+    my @opt;
+    push @opt, fork => 1 if $$ != $$self->{-pid};
+
+    $self->_cleanup(@opt);
+}
+
+sub _cleanup {
+    my $self = ${ +shift };
+    my %opt = @_;
     local $self->{-cleanup} = 1; # This is stronger than lock.
 
     # NOTE Be careful! cleanup must never ever die!
@@ -406,21 +431,11 @@ sub cleanup {
         $spec->{$a}{cleanup_order} <=> $spec->{$b}{cleanup_order};
     } keys %{ $self->{-cache} };
 
+
     foreach my $name (@order) {
-        local $@; # don't pollute $@ if we're in destructor after an exception
-        eval {
-            # We cannot afford to die here as if we do
-            #    a resource that causes exceptions in cleanup
-            #    would be stuck in cache forever
-            $self->_silo_cleanup_res($name);
-            1;
-        } or do {
-            my $err = $@;
-            Carp::cluck "Failed to cleanup resource '$name', but trying to continue: $err";
-        };
+        $self->_silo_cleanup_res($name, %opt);
     };
 
-    delete $self->{-cache};
     return $_[0];
 };
 
